"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const workspace_1 = require("@nrwl/workspace");
const ast_utils_1 = require("@nrwl/workspace/src/utils/ast-utils");
const visitors_1 = require("../../common/utils/visitors");
const changes_1 = require("../../common/utils/changes");
const renames = {
    LetModule: '@rx-angular/template/let',
    LetDirective: '@rx-angular/template/let',
    PushModule: '@rx-angular/template/push',
    PushPipe: '@rx-angular/template/push',
    UnpatchDirective: '@rx-angular/template/unpatch',
    UnpatchModule: '@rx-angular/template/unpatch',
};
function default_1() {
    return (tree) => {
        visitors_1.visitTSSourceFiles(tree, (sourceFile, tree) => {
            const imports = sourceFile.statements
                .filter(ts.isImportDeclaration)
                .filter(({ moduleSpecifier }) => moduleSpecifier.getText(sourceFile) === `'@rx-angular/template'` ||
                moduleSpecifier.getText(sourceFile) === `"@rx-angular/template"`);
            if (imports.length === 0) {
                return;
            }
            const changes = updateTemplateImportDeclarations(sourceFile, imports);
            ast_utils_1.insert(tree, sourceFile.fileName, changes);
        });
    };
}
exports.default = default_1;
function updateTemplateImportDeclarations(sourceFile, imports) {
    return imports
        .flatMap((importDeclaration) => {
        const importSpecifiers = workspace_1.findNodes(importDeclaration, ts.SyntaxKind.ImportSpecifier);
        return importSpecifiers.map((importSpecifier) => ({
            importDeclaration,
            importSpecifier: importSpecifier.getText(),
        }));
    })
        .flatMap(({ importDeclaration, importSpecifier }) => {
        return [
            changes_1.createRemoveChange(sourceFile, importDeclaration),
            ast_utils_1.insertImport(sourceFile, sourceFile.fileName, importSpecifier, renames[importSpecifier])
        ];
    });
}
